"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
// Import intercom
const react_intercom_1 = __importDefault(require("react-intercom"));
const gridData_1 = require("../utils/gridData");
// Import helpers
const gridStyling_1 = require("../utils/gridStyling");
// Import sheet and code components
const MitoSheet_1 = __importDefault(require("./MitoSheet"));
const SheetTabs_1 = __importDefault(require("./tabs/SheetTabs"));
const FormulaBar_1 = __importDefault(require("./FormulaBar"));
const MitoToolbar_1 = __importDefault(require("./MitoToolbar"));
const LoadingIndicator_1 = __importDefault(require("./LoadingIndicator"));
// Import modals
const ErrorModal_1 = __importDefault(require("./ErrorModal"));
const DownloadModal_1 = __importDefault(require("./DownloadModal"));
const ColumnHeaderModal_1 = __importDefault(require("./modals/ColumnHeaderModal"));
const FilterModal_1 = __importDefault(require("./modals/FilterModal"));
const DeleteColumnModal_1 = __importDefault(require("./modals/DeleteColumnModal"));
const SaveAnalysisModal_1 = __importDefault(require("./modals/SaveAnalysisModal"));
const SaveAnalysisOverwriteModal_1 = __importDefault(require("./modals/SaveAnalysisOverwriteModal"));
const ReplayAnalysisModal_1 = __importDefault(require("./modals/ReplayAnalysisModal"));
const ImportModal_1 = __importDefault(require("./modals/ImportModal"));
// Import css
require("../../css/mito.css");
const ReplayImportModal_1 = __importDefault(require("./modals/ReplayImportModal"));
const PivotTaskpane_1 = __importDefault(require("./taskpanes/PivotTable/PivotTaskpane"));
const IntroductionTour_1 = __importDefault(require("./tours/IntroductionTour"));
const taskpanes_1 = require("./taskpanes/taskpanes");
const MergeTaskpane_1 = __importDefault(require("./taskpanes/Merge/MergeTaskpane"));
const DocumentationTaskpane_1 = __importDefault(require("./taskpanes/Documentation/DocumentationTaskpane"));
var ModalEnum;
(function (ModalEnum) {
    ModalEnum["None"] = "None";
    ModalEnum["Error"] = "Error";
    ModalEnum["RepeatAnalysis"] = "RepeatAnalysis";
    ModalEnum["Download"] = "Download";
    ModalEnum["ColumnHeader"] = "ColumnHeader";
    ModalEnum["DeleteColumn"] = "DeleteColumn";
    ModalEnum["SaveAnalysis"] = "SaveAnalysis";
    ModalEnum["SaveAnalysisOverwrite"] = "SaveAnalysisOverwrite";
    ModalEnum["ReplayAnalysis"] = "ReplayAnalysis";
    ModalEnum["Filter"] = "Filter";
    ModalEnum["Import"] = "Import";
    ModalEnum["ReplayImport"] = "ReplayImport";
})(ModalEnum = exports.ModalEnum || (exports.ModalEnum = {}));
class Mito extends react_1.default.Component {
    constructor(props) {
        super(props);
        let formulaBarValue = '';
        let selectedColumn = '';
        // Set the defaul values for the formula bar and selected column (if possible, aka the sheet has some data in it)
        if (this.props.sheetJSONArray.length > 0) {
            formulaBarValue = this.props.sheetJSONArray[0].data[0] === undefined
                ? '' : this.props.sheetJSONArray[0].data[0][0];
            selectedColumn = this.props.sheetJSONArray[0].columns[0] === undefined
                ? '' : this.props.sheetJSONArray[0].columns[0].toString();
        }
        // Build the default column widths
        const defaultColumnWidthsArray = gridStyling_1.getColumnWidthsArray(this.props.sheetJSONArray).columnWidthsArray;
        this.state = {
            tutorialMode: props.tutorialMode,
            runTour: false,
            tourStepIndex: 0,
            dfNames: this.props.dfNames,
            sheetShapeArray: this.props.sheetShapeArray,
            columnSpreadsheetCodeJSONArray: this.props.columnSpreadsheetCodeJSONArray,
            sheetJSONArray: this.props.sheetJSONArray,
            columnFiltersArray: this.props.columnFiltersArray,
            columnTypeJSONArray: this.props.columnTypeJSONArray,
            selectedSheetIndex: 0,
            formulaBarValue: formulaBarValue,
            selectedColumn: selectedColumn,
            selectedRowIndex: 0,
            /*
                we tell if we're in cell editing mode through the editingModeOn state variable

                the editingCellColumn and editingCellRowIndex are used to update the formula of the correct
                cell editor. They get set to the new values when we turn cell editing mode back on, but are unchanged
                when we turn cell editing mode off so that we can handle formulas that error by letting the user
                edit the formula that they submitted.
            */
            editingModeState: {
                editingModeOn: false,
                editingSheetIndex: 0,
                editingCellColumn: "",
                editingCellRowIndex: -1,
                lastFormula: formulaBarValue,
                formulaCursorIndex: 2,
            },
            movingColumnParams: undefined,
            documentationOpen: false,
            errorJSON: {
                event: '',
                type: '',
                header: '',
                to_fix: ''
            },
            modalInfo: { type: ModalEnum.None },
            savedAnalysisNames: this.props.savedAnalysisNames,
            loading: false,
            columnWidthsArray: defaultColumnWidthsArray,
            isLocalDeployment: props.isLocalDeployment,
            currOpenTaskpane: { type: taskpanes_1.TaskpaneType.NONE },
        };
        this.cellFocused = this.cellFocused.bind(this);
        this.handleFormulaBarDoubleClick = this.handleFormulaBarDoubleClick.bind(this);
        this.sendCellValueUpdate = this.sendCellValueUpdate.bind(this);
        this.setEditingMode = this.setEditingMode.bind(this);
        this.setEditingFormula = this.setEditingFormula.bind(this);
        this.setModal = this.setModal.bind(this);
        this.getCurrentModalComponent = this.getCurrentModalComponent.bind(this);
        this.setSelectedSheetIndex = this.setSelectedSheetIndex.bind(this);
        this.turnOnCellEditor = this.turnOnCellEditor.bind(this);
        this.setCursorIndex = this.setCursorIndex.bind(this);
        this.columnMoved = this.columnMoved.bind(this);
        this.columnDragStopped = this.columnDragStopped.bind(this);
        this.setColumnWidth = this.setColumnWidth.bind(this);
        this.setCurrOpenTaskpane = this.setCurrOpenTaskpane.bind(this);
        this.getCurrOpenTaskpane = this.getCurrOpenTaskpane.bind(this);
        this.closeEditingTaskpane = this.closeEditingTaskpane.bind(this);
        this.setTourInfo = this.setTourInfo.bind(this);
    }
    /*
        if we're in tutorial mode, start the tour .5 seconds after page loading
        so that the tour selector has items to select. This also looks nice :)
    */
    componentDidMount() {
        setTimeout(() => {
            if (this.state.tutorialMode == true) {
                this.setState({ runTour: true });
            }
        }, 500);
    }
    /*
        Registers the step index with the Mito state so that when the
        tour is recreated because the key updates, we don't start back at the beginning
    */
    setTourInfo(runTour, newStepIndex) {
        this.setState({
            tourStepIndex: newStepIndex,
            runTour: runTour,
            tutorialMode: runTour
        }, () => {
            var _a;
            // Log the step advancement creation
            (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
                userId: window.user_id,
                event: 'Introduction Tour',
                properties: {
                    tour_id: 'Introduction Tour',
                    stepIndex: newStepIndex,
                    runTour: runTour
                }
            });
        });
    }
    /*
        The componentDidUpdate hook is responsible for maintaining consistency in the state
        variables of this prop in the case of updates to the sheet data. NOTE: this consistency
        should enforce that any column you index into should be defined for all sheets + all columns
        (e.g. this.state.columnFilters should be defined everywhere). If this is not the case, then it MUST
        be enforced in the type system!

        For example, if a sheet is deleted, then selectedSheetIndex in that state should not
        be set equal to the old index of that sheet.

        See more here: https://reactjs.org/docs/react-component.html#componentdidupdate

        NOTE: careful on calling this.setState in this function, as repeated calls cause
        the sheet to crash (as this recurses infinitely).

        NOTE: this function will fail if it is called on a dataframe with mulitple columns with
        the same name. This is an invariant we currently expect to be the case.
    */
    componentDidUpdate(prevProps, prevState) {
        const newSheetJSON = this.state.sheetJSONArray;
        let newSelectedSheetIndex = prevState.selectedSheetIndex;
        // If the selectedSheetIndex no longer exists, because of an undo, don't select that one!
        // Or, if there was an a new sheet creation, then default to the last sheet
        if (prevState.selectedSheetIndex >= newSheetJSON.length || prevState.sheetJSONArray.length < newSheetJSON.length) {
            newSelectedSheetIndex = newSheetJSON.length - 1;
        }
        if (newSelectedSheetIndex != prevState.selectedSheetIndex) {
            this.setState({
                selectedSheetIndex: newSelectedSheetIndex
            });
        }
        // Make sure the column widths are set correctly, which they may not be if the columns in sheet
        // have changed, or a sheet has been added or subtracted
        const columnWidthsArrayResult = gridStyling_1.getColumnWidthsArray(this.state.sheetJSONArray, this.state.columnWidthsArray);
        // Check if we need to actually update the column widths (sheet number change, any column change)
        if (columnWidthsArrayResult.changed) {
            this.setState({
                columnWidthsArray: columnWidthsArrayResult.columnWidthsArray
            });
        }
    }
    /*
        Called by the cell editor so that the column the user clicks on can be appended
        to the correct location in the formula
    */
    setCursorIndex(index) {
        this.setState(prevState => {
            return {
                editingModeState: Object.assign(Object.assign({}, prevState.editingModeState), { formulaCursorIndex: index })
            };
        });
    }
    /*
        This function is responsible for turning cell editing mode on and off
        by setting the state of editingCellColumn, and editingCellRowIndex.

        If you call this function with on===true, then the passed column and rowIndex
        should be the cell you want to start editing.

        If you call this function with on===false, then editing will be stopped, and the
        passed column and rowIndex will be focused on.
    */
    setEditingMode(on, column, rowIndex) {
        var _a, _b, _c, _d;
        if (on && !this.state.editingModeState.editingModeOn) {
            // we close the taskpane if its open and its a editing taskpane
            this.closeEditingTaskpane();
            /*
                This runs (and turns on editing mode) when we're not in editing mode and:
                1. The user double clicks on a cell
                2. The user presses enter on a cell.
                3. The user types any character on a cell.
            */
            this.setState(prevState => {
                return {
                    editingModeState: Object.assign(Object.assign({}, prevState.editingModeState), { editingSheetIndex: this.state.selectedSheetIndex, editingModeOn: true, editingCellColumn: column, editingCellRowIndex: rowIndex })
                };
            });
        }
        else if (!on) {
            /*
                We turn off cell-editing mode and select the given cell.

                This handles some of the many ways cell editing can be stopped
                explicitly (e.g. ENTER), as we want only sometimes want ENTER to stop
                editing (in other cases, we want it to select a suggestion!).

                To see a list of events that stop editing, see:
                https://www.ag-grid.com/javascript-grid-cell-editing/#stop-end-editing
            */
            this.setState(prevState => {
                return {
                    editingModeState: Object.assign(Object.assign({}, prevState.editingModeState), { editingModeOn: false })
                };
            });
            // We actually stop the grid from editing in this case, and set cell focus
            // as stopping editing focuses on nothing by default.
            (_b = (_a = window.gridApiMap) === null || _a === void 0 ? void 0 : _a.get(this.props.model_id)) === null || _b === void 0 ? void 0 : _b.stopEditing();
            (_d = (_c = window.gridApiMap) === null || _c === void 0 ? void 0 : _c.get(this.props.model_id)) === null || _d === void 0 ? void 0 : _d.setFocusedCell(rowIndex, column);
        }
    }
    /*
        This function is called by the cell editor to update the formula of the editing cell
        in the mito state, so that when cellFocused starts editing mode again, it can
        pass in the starting formula.

        We save the formula in the state because:
            1) when the user clicks on another cell, it closes the cell editor.
            We need to store the formula in state so when we put the editor
            back in cell editing mode in the CellFocussedEvent,
            we're able to start the cell editor with the correct formula.
            
            Note: we don't want to send the intermediate formula to the parser

            2) we need to display the formula that is in the editor in the formula bar as well.
    */
    setEditingFormula(formula) {
        this.setState(prevState => {
            return {
                editingModeState: Object.assign(Object.assign({}, prevState.editingModeState), { lastFormula: formula }),
                formulaBarValue: formula,
            };
        });
    }
    turnOnCellEditor() {
        var _a, _b, _c, _d;
        const editingModeParams = {
            rowIndex: this.state.editingModeState.editingCellRowIndex,
            colKey: this.state.editingModeState.editingCellColumn,
        };
        // Scroll to make sure this editor is visible (but only scroll just enough)
        (_b = (_a = window.gridApiMap) === null || _a === void 0 ? void 0 : _a.get(this.props.model_id)) === null || _b === void 0 ? void 0 : _b.ensureColumnVisible(editingModeParams.colKey);
        // TODO: we might want to ensureIndexVisible as well, but users always edit at the top
        // turn the editing cell's cell editor back on!
        (_d = (_c = window.gridApiMap) === null || _c === void 0 ? void 0 : _c.get(this.props.model_id)) === null || _d === void 0 ? void 0 : _d.startEditingCell(editingModeParams);
    }
    /*
        Occurs when a cell is clicked. If we are currently editing a cell, we append the clicked
        column to the currently-edited cell. Otherwise, we select the clicked cell.
    */
    cellFocused(event) {
        var _a;
        /*
            We avoid cell focused throwing an error when switching sheets by making sure the
            event is defined, and just returning. Not sure why this occurs!

            Additionally, turn of cell editing mode & reset the formulaBarValue. when the user switches
            sheets we reset the editing cell to clear the formula bar
        */
        const column = (_a = event.column) === null || _a === void 0 ? void 0 : _a.getColId();
        const rowIndex = event.rowIndex;
        if (!column) {
            this.setEditingMode(false, "", -1);
            this.setState(prevState => {
                return {
                    editingModeState: Object.assign(Object.assign({}, prevState.editingModeState), { editingCellColumn: '', editingCellRowIndex: -1 }),
                    formulaBarValue: '',
                };
            });
            return;
        }
        // If we're in cell editing mode, turn cell editor back on
        if (this.state.editingModeState.editingModeOn) {
            /*
                If the column we focussed on is the editing column or the index column, don't append to the formula
            */
            if (column === this.state.editingModeState.editingCellColumn || column === gridData_1.MITO_INDEX) {
                this.turnOnCellEditor();
                return;
            }
            /*
                If we're in editing mode & the cell focused on is not in the column we're editing
                or the index column, append the clicked column to the editing formula;
            */
            // Add the column reference to the formula where the user's cursor is in the input field
            const formulaCursorIndex = this.state.editingModeState.formulaCursorIndex;
            // If you click on the same row, we just insert that column name. If you click on a different
            // row than the one you are editing, we insert an offset formula.
            const columnReference = this.state.editingModeState.editingCellRowIndex === rowIndex
                ? column
                : `OFFSET(${column}, ${rowIndex - this.state.editingModeState.editingCellRowIndex})`;
            const newFormula = this.state.formulaBarValue.slice(0, formulaCursorIndex) + columnReference + this.state.formulaBarValue.slice(formulaCursorIndex);
            this.setState(prevState => {
                return {
                    editingModeState: Object.assign(Object.assign({}, prevState.editingModeState), { lastFormula: newFormula, formulaCursorIndex: formulaCursorIndex + columnReference.length // update the formula cursor index to include appended characters
                     }),
                    formulaBarValue: newFormula,
                };
            }, () => {
                this.turnOnCellEditor();
            });
        }
        else {
            // If we're not in editing mode, then we update the formula bar only
            // If the column is the index column, then we reset the selected cell state
            if (column === gridData_1.MITO_INDEX) {
                this.setState({
                    selectedColumn: '',
                    selectedRowIndex: 0,
                    formulaBarValue: ''
                });
                return;
            }
            // Otherwise, get the cell's formula to display
            const columnIndex = this.state.sheetJSONArray[this.state.selectedSheetIndex].columns.indexOf(column);
            const rowIndex = event.rowIndex;
            const columnFormula = this.state.columnSpreadsheetCodeJSONArray[this.state.selectedSheetIndex][column];
            let formulaBarValue = this.state.formulaBarValue;
            if (columnFormula === '') {
                // If there no formula, we just display the value
                formulaBarValue = this.state.sheetJSONArray[this.state.selectedSheetIndex].data[rowIndex][columnIndex];
            }
            else {
                // Otherwise, we display the formula that is in the column
                formulaBarValue = columnFormula;
                if (this.state.selectedSheetIndex === this.state.editingModeState.editingSheetIndex && column === this.state.editingModeState.editingCellColumn) {
                    // The last formula is always equal to the _last_ edit made to a formula
                    // and so we take it so we keep an error formula
                    formulaBarValue = this.state.editingModeState.lastFormula;
                }
            }
            this.setState({
                selectedColumn: column,
                selectedRowIndex: rowIndex,
                formulaBarValue: formulaBarValue
            });
        }
    }
    handleFormulaBarDoubleClick(e) {
        var _a, _b;
        e.preventDefault();
        /*
            When the formula bar is double clicked, turns on cell editing mode and opens the
            selected cell's cell editor

            TODO: make sure it works with multiple sheets. fix: when we switch sheets,
            reset this.state.selectedColumn & this.state.selectedRowIndex
        */
        // Do nothing if the selected cell is a data column
        if (this.state.columnSpreadsheetCodeJSONArray[this.state.selectedSheetIndex][this.state.selectedColumn] === '') {
            return;
        }
        // Turn cell editing mode on
        this.setEditingMode(true, this.state.selectedColumn, this.state.selectedRowIndex);
        /*
            Turns on the correct cell editor. If the selected cell is not in view,
            the spreadsheet will be scrolled automatically until it is.
        */
        const editingModeParams = {
            rowIndex: this.state.selectedRowIndex,
            colKey: this.state.selectedColumn,
        };
        (_b = (_a = window.gridApiMap) === null || _a === void 0 ? void 0 : _a.get(this.props.model_id)) === null || _b === void 0 ? void 0 : _b.startEditingCell(editingModeParams);
    }
    // TODO: this event should be broken out into a formula edit and a value edit
    sendCellValueUpdate(column, newValue) {
        /*
            We don't send the formula to the evaluator while in cell editing mode
            because this function gets called after the CellValueChangedEvent fires
            each time the cell editor is closed.
            
            However, the cell editor closes each time the user uses their mouse
            to reference another column - which isn't a finished update yet!
        */
        if (!this.state.editingModeState.editingModeOn) {
            this.props.send({
                'event': 'edit_event',
                'type': 'set_column_formula_edit',
                'sheet_index': this.state.selectedSheetIndex,
                'column_header': column,
                'new_formula': newValue
            });
        }
    }
    getCurrentModalComponent() {
        // Returns the JSX.element that is currently, open, and is used
        // in render to display this modal
        switch (this.state.modalInfo.type) {
            case ModalEnum.None: return react_1.default.createElement("div", null);
            case ModalEnum.Error: return (react_1.default.createElement(ErrorModal_1.default, { errorJSON: this.state.errorJSON, setModal: this.setModal }));
            case ModalEnum.Download: return (react_1.default.createElement(DownloadModal_1.default, { setModal: this.setModal }));
            case ModalEnum.ColumnHeader: return (react_1.default.createElement(ColumnHeaderModal_1.default, { setModal: this.setModal, send: this.props.send, columnHeader: this.state.modalInfo.columnHeader, selectedSheetIndex: this.state.selectedSheetIndex }));
            case ModalEnum.Filter: return (react_1.default.createElement(FilterModal_1.default, { selectedSheetIndex: this.state.selectedSheetIndex, setModal: this.setModal, columnHeader: this.state.modalInfo.columnHeader, columnType: this.state.columnTypeJSONArray[this.state.selectedSheetIndex][this.state.modalInfo.columnHeader], send: this.props.send, operator: this.state.columnFiltersArray[this.state.selectedSheetIndex][this.state.modalInfo.columnHeader].operator, filters: this.state.columnFiltersArray[this.state.selectedSheetIndex][this.state.modalInfo.columnHeader].filters }));
            case ModalEnum.DeleteColumn: return (react_1.default.createElement(DeleteColumnModal_1.default, { setModal: this.setModal, send: this.props.send, columnHeader: this.state.modalInfo.columnHeader, selectedSheetIndex: this.state.selectedSheetIndex }));
            case ModalEnum.SaveAnalysis: return (react_1.default.createElement(SaveAnalysisModal_1.default, { setModal: this.setModal, send: this.props.send, savedAnalysisNames: this.state.savedAnalysisNames, saveAnalysisName: this.state.modalInfo.saveAnalysisName }));
            case ModalEnum.SaveAnalysisOverwrite: return (react_1.default.createElement(SaveAnalysisOverwriteModal_1.default, { setModal: this.setModal, send: this.props.send, savedAnalysisNames: this.state.savedAnalysisNames, saveAnalysisName: this.state.modalInfo.saveAnalysisName }));
            case ModalEnum.ReplayAnalysis: return (react_1.default.createElement(ReplayAnalysisModal_1.default, { setModal: this.setModal, send: this.props.send, savedAnalysisNames: this.state.savedAnalysisNames, api: this.props.mitoAPI }));
            case ModalEnum.Import: return (react_1.default.createElement(ImportModal_1.default, { setModal: this.setModal, send: this.props.send, dfNames: this.state.dfNames, api: this.props.mitoAPI }));
            case ModalEnum.ReplayImport: return (react_1.default.createElement(ReplayImportModal_1.default, { setModal: this.setModal, send: this.props.send, dfNames: this.state.dfNames, api: this.props.mitoAPI, analysisName: this.state.modalInfo.analysisName, importSummaries: this.state.modalInfo.importSummary }));
        }
    }
    setModal(modalInfo) {
        // Sets the current modal. We _always_ turn off cell editing mode here, just
        // in case the user opens/closes a modal in cell editing mode.
        this.setEditingMode(false, "", -1);
        // unless opening an error modal or closing a modal, close open editing taskpanes
        if (modalInfo.type !== ModalEnum.Error && modalInfo.type !== ModalEnum.None) {
            this.closeEditingTaskpane();
        }
        // And then we actually update the modalInfo
        this.setState({ modalInfo: modalInfo });
    }
    /*
        Returns the JSX.Element of the taskpane that should be currently
        displayed, stored in this.state.currOpenTaskpane.

        If this.state.currOpenTaskpane is undefined, then returns no taskpane
        at all.
    */
    getCurrOpenTaskpane() {
        switch (this.state.currOpenTaskpane.type) {
            case taskpanes_1.TaskpaneType.NONE: return (react_1.default.createElement(react_1.Fragment, null));
            case taskpanes_1.TaskpaneType.PIVOT: return (react_1.default.createElement(PivotTaskpane_1.default, { dfNames: this.state.dfNames, sheetJSONArray: this.state.sheetJSONArray, send: this.props.send, selectedSheetIndex: this.state.selectedSheetIndex, setCurrOpenTaskpane: this.setCurrOpenTaskpane }));
            case taskpanes_1.TaskpaneType.MERGE: return (react_1.default.createElement(MergeTaskpane_1.default, { dfNames: this.state.dfNames, sheetJSONArray: this.state.sheetJSONArray, send: this.props.send, setCurrOpenTaskpane: this.setCurrOpenTaskpane }));
            case taskpanes_1.TaskpaneType.DOCUMENTATION: return (react_1.default.createElement(DocumentationTaskpane_1.default, { setCurrOpenTaskpane: this.setCurrOpenTaskpane }));
        }
    }
    /*
        Changes what taskpane is currently displayed. Set to undefined
        to close the open taskpane.
    */
    setCurrOpenTaskpane(newTaskpaneInfo) {
        this.setState({ currOpenTaskpane: newTaskpaneInfo });
    }
    /*
        Closes Editing Taskpanes and leaves other taskpanes open
    */
    closeEditingTaskpane() {
        if (taskpanes_1.EDITING_TASKPANES.includes(this.state.currOpenTaskpane.type)) {
            this.setState({
                currOpenTaskpane: { type: taskpanes_1.TaskpaneType.NONE }
            });
        }
    }
    setSelectedSheetIndex(newIndex) {
        // We turn off editing mode, if it is on, because we moved to a new sheet
        this.setEditingMode(false, "", -1);
        // We use the backend to log that we switched sheets
        this.props.send({
            'event': 'log_event',
            'type': 'switch_sheet_log_event',
            'sheet_index': newIndex
        });
        // And then we update the selected index
        this.setState({ selectedSheetIndex: newIndex });
    }
    /*
        The ColumnMovedEvent gets fired when a user drags a column to change its location.
        While dragging the column from one position to another, this event gets fired multiple times.
        When a column is moving, we set the movingColumnParams so we can send a reorder event to the backend
    */
    columnMoved(event) {
        var _a, _b;
        this.setState({
            movingColumnParams: {
                column: ((_a = event.column) === null || _a === void 0 ? void 0 : _a.getColId()) ? (_b = event.column) === null || _b === void 0 ? void 0 : _b.getColId() : '',
                newIndex: event.toIndex ? event.toIndex : -1 // make sure type checks
            }
        });
    }
    // When the column stops moving, send the column reorder event to the backend 
    columnDragStopped() {
        var _a;
        /*
            The DragStoppedEvent gets fired when the user changes the width of the column and when
            the user reorders the column. We only want to send a column_reorder_event when the user is
            reordering the column, so we check if the user is moving a column by checking this.state.columnMovedState.column
        */
        if (this.state.movingColumnParams != undefined && this.state.movingColumnParams.column !== '') {
            // If the column did not change locations, don't send a column_reorder_event
            if (this.state.sheetJSONArray[this.state.selectedSheetIndex].columns[this.state.movingColumnParams.newIndex - 1] === this.state.movingColumnParams.column) {
                return;
            }
            this.props.send({
                'event': 'edit_event',
                'type': 'reorder_column_edit',
                'sheet_index': this.state.selectedSheetIndex,
                'column_header': this.state.movingColumnParams.column,
                'new_column_index': this.state.movingColumnParams.newIndex - 1 // - 1 to account for the index column
            });
            // Log the column reorder event
            (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
                userId: window.user_id,
                event: 'column_reoder_event',
                properties: {
                    column_header: this.state.movingColumnParams.column,
                    new_column_index: this.state.movingColumnParams.newIndex - 1
                }
            });
            // Reset the columnMovedState so a column resizing doesn't cause the column_reorder_event to get sent
            this.setState({
                movingColumnParams: undefined
            });
        }
    }
    setColumnWidth(sheetIndex, columnHeader, columnWidth) {
        this.setState(prevState => {
            const newColumnsWidthsArray = [...prevState.columnWidthsArray];
            newColumnsWidthsArray[sheetIndex][columnHeader] = columnWidth;
            return Object.assign(Object.assign({}, prevState), { columnWidthsArray: newColumnsWidthsArray });
        });
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    render() {
        // We get the class to open or close the taskpane, as we open the taskpane by changing
        // the widths of the divs in the css
        const mitoFormulaBarAndMitosheetClass = this.state.currOpenTaskpane.type !== taskpanes_1.TaskpaneType.NONE ?
            'mito-formula-bar-and-mitosheet-div mito-formula-bar-and-mitosheet-div-taskpane-open' :
            'mito-formula-bar-and-mitosheet-div';
        const mitoDefaultTaskpaneClass = this.state.currOpenTaskpane.type !== taskpanes_1.TaskpaneType.NONE ?
            'mito-default-taskpane mito-default-taskpane-open' :
            'mito-default-taskpane';
        return (react_1.default.createElement("div", { className: "mito-container", ref: (ref) => { this.mitoContainerRef = ref; } },
            react_1.default.createElement("div", { className: "mitosheet" },
                react_1.default.createElement(MitoToolbar_1.default, { sheetJSON: this.state.sheetJSONArray[this.state.selectedSheetIndex], selectedSheetIndex: this.state.selectedSheetIndex, send: this.props.send, setEditingMode: this.setEditingMode, setModal: this.setModal, model_id: this.props.model_id, selectedColumn: this.state.selectedColumn, setCurrOpenTaskpane: this.setCurrOpenTaskpane, closeEditingTaskpane: this.closeEditingTaskpane }),
                react_1.default.createElement("div", { className: "mito-main-sheet-div" },
                    react_1.default.createElement("div", { className: mitoFormulaBarAndMitosheetClass },
                        this.state.runTour &&
                            react_1.default.createElement(IntroductionTour_1.default
                            // Set key so IntroductionTour gets created with new state when tour is turned on/off or import modal is open/closed
                            // see comment in IntroductionTour constructor for more info
                            , { 
                                // Set key so IntroductionTour gets created with new state when tour is turned on/off or import modal is open/closed
                                // see comment in IntroductionTour constructor for more info
                                key: this.state.runTour + this.state.modalInfo.type.toString(), isImportModalOpen: this.state.modalInfo.type == ModalEnum.Import, run: this.state.runTour, stepIndex: this.state.tourStepIndex, setTourInfo: this.setTourInfo }),
                        react_1.default.createElement(FormulaBar_1.default, { formulaBarValue: this.state.formulaBarValue, handleFormulaBarDoubleClick: this.handleFormulaBarDoubleClick }),
                        react_1.default.createElement(MitoSheet_1.default, { selectedSheetIndex: this.state.selectedSheetIndex, formulaBarValue: this.state.formulaBarValue, editingColumn: this.state.editingModeState.editingCellColumn, columnSpreadsheetCodeJSON: this.state.columnSpreadsheetCodeJSONArray[this.state.selectedSheetIndex], sheetJSON: this.state.sheetJSONArray[this.state.selectedSheetIndex], columnFiltersJSON: this.state.columnFiltersArray[this.state.selectedSheetIndex], setEditingMode: this.setEditingMode, setModal: this.setModal, setEditingFormula: this.setEditingFormula, setCursorIndex: this.setCursorIndex, cursorIndex: this.state.editingModeState.formulaCursorIndex, cellFocused: this.cellFocused, columnMoved: this.columnMoved, columnDragStopped: this.columnDragStopped, model_id: this.props.model_id, sendCellValueUpdate: this.sendCellValueUpdate, columnWidths: this.state.columnWidthsArray[this.state.selectedSheetIndex], setColumnWidth: this.setColumnWidth })),
                    this.state.currOpenTaskpane.type !== taskpanes_1.TaskpaneType.NONE &&
                        react_1.default.createElement("div", { className: mitoDefaultTaskpaneClass }, this.getCurrOpenTaskpane()))),
            react_1.default.createElement(SheetTabs_1.default, { mitoContainerRef: this.mitoContainerRef, send: this.props.send, dfNames: this.state.dfNames, sheetShapeArray: this.state.sheetShapeArray, selectedSheetIndex: this.state.selectedSheetIndex, setSelectedSheetIndex: this.setSelectedSheetIndex, closeEditingTaskpane: this.closeEditingTaskpane }),
            this.getCurrentModalComponent(),
            this.state.loading && react_1.default.createElement(LoadingIndicator_1.default, null),
            !this.state.isLocalDeployment && react_1.default.createElement(react_intercom_1.default, { appID: 'mu6azgiv' })));
    }
}
exports.default = Mito;
//# sourceMappingURL=Mito.js.map