"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const DefaultTaskpane_1 = __importDefault(require("../DefaultTaskpane"));
const LargeSelect_1 = __importDefault(require("../../elements/LargeSelect"));
const PivotTableKeySelection_1 = __importDefault(require("./PivotTableKeySelection"));
// Import 
require("../../../../css/pivot-taskpane.css");
const PivotTableValueSelection_1 = __importDefault(require("./PivotTableValueSelection"));
// The aggregation functions we're supporting, out of the box
var AggregationType;
(function (AggregationType) {
    AggregationType["SUM"] = "sum";
    AggregationType["MEAN"] = "mean";
    AggregationType["MEDIAN"] = "median";
    AggregationType["MIN"] = "min";
    AggregationType["MAX"] = "max";
    AggregationType["COUNT"] = "count";
    AggregationType["STD"] = "std";
})(AggregationType = exports.AggregationType || (exports.AggregationType = {}));
var KeyType;
(function (KeyType) {
    KeyType["Row"] = "Row";
    KeyType["Col"] = "Col";
})(KeyType = exports.KeyType || (exports.KeyType = {}));
var SelectionOperation;
(function (SelectionOperation) {
    SelectionOperation["ADD"] = "Add";
    SelectionOperation["REMOVE"] = "Remove";
})(SelectionOperation = exports.SelectionOperation || (exports.SelectionOperation = {}));
class PivotTaskpane extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.editValueAggregationSelection = (columnHeader, operation, aggregationType) => {
            this.setState(prevState => {
                const newValues = JSON.parse(JSON.stringify(prevState.values));
                if (operation == SelectionOperation.ADD && aggregationType) {
                    // If the column is being added as a new aggregated value, add it
                    newValues[columnHeader] = aggregationType;
                }
                else if (operation == SelectionOperation.REMOVE) {
                    // If the column is being removed as an aggregated value, remove it
                    delete newValues[columnHeader];
                }
                return {
                    values: newValues
                };
            }, () => {
                this.sendPivotTableUpdateMessage();
            });
        };
        /*
            A callback used by the select data source Select Element so that it can
            set the state of the Pivot Table Taskpane
        */
        this.setSelectedSheet = (newSheetName) => {
            const newSelectedSheetIndex = this.state.dfNames.indexOf(newSheetName);
            // If you didn't select a new sheet, then don't do clear your selections
            if (newSelectedSheetIndex == this.state.selectedSheetIndex) {
                return;
            }
            /*
                Reset the keys so
                    1) the column headers currently selected are dropped
                    2) the values currently aggregated are dropped
                    3) display the column headers from the new sheet
            */
            this.setState({
                selectedSheetIndex: newSelectedSheetIndex,
                selectedGroupByRows: [],
                selectedGroupByCols: [],
                values: {}
            });
        };
        // Function used to add/remove column headers to the row and column keys 
        this.editKeySelection = (keyType, columnHeader, operation) => {
            // Sanity check: make sure that this column is not already selected. this should not be possible!
            if (operation === SelectionOperation.ADD) {
                if (this.state.selectedGroupByRows.includes(columnHeader) || this.state.selectedGroupByCols.includes(columnHeader)) {
                    console.log("column selected as key twice in pivot table. should not be possible");
                    return;
                }
            }
            this.setState(prevState => {
                let previousColumnHeaders = [];
                if (keyType === KeyType.Row) {
                    previousColumnHeaders = [...prevState.selectedGroupByRows];
                }
                else {
                    previousColumnHeaders = [...prevState.selectedGroupByCols];
                }
                if (operation == SelectionOperation.ADD && !previousColumnHeaders.includes(columnHeader)) {
                    // If the operation was add, add the column header to the list of column headers (if not already included)
                    previousColumnHeaders.push(columnHeader);
                }
                else if (operation == SelectionOperation.REMOVE && previousColumnHeaders.includes(columnHeader)) {
                    // If the operation was remove, add the column header to the list of column headers (if not already included)
                    previousColumnHeaders.splice(previousColumnHeaders.indexOf(columnHeader), 1);
                }
                return Object.assign(Object.assign({}, prevState), { [keyType === KeyType.Row ? 'selectedGroupByRows' : 'selectedGroupByCols']: previousColumnHeaders });
            }, () => {
                this.sendPivotTableUpdateMessage();
            });
        };
        /*
            Completes the group operation by sending information for the group
            to the backend.
        */
        this.sendPivotTableUpdateMessage = (overwrite = true) => {
            var _a;
            // Log
            (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
                userId: window.user_id,
                event: 'button_pivot_log_event',
                properties: {
                    sheet_index: this.state.selectedSheetIndex,
                    pivot_rows: this.state.selectedGroupByRows,
                    pivot_cols: this.state.selectedGroupByCols,
                    values: this.state.values,
                    overwrite: overwrite
                }
            });
            this.props.send({
                'event': 'edit_event',
                'type': 'pivot_edit',
                sheet_index: this.state.selectedSheetIndex,
                pivot_rows: this.state.selectedGroupByRows,
                pivot_columns: this.state.selectedGroupByCols,
                values: this.state.values,
                overwrite: overwrite
            });
        };
        this.state = {
            dfNames: this.props.dfNames,
            selectedSheetIndex: props.selectedSheetIndex,
            selectedGroupByRows: [],
            selectedGroupByCols: [],
            values: {},
            keyError: '',
        };
        this.getSelectableKeyColumnHeaders = this.getSelectableKeyColumnHeaders.bind(this);
        this.editValueAggregationSelection = this.editValueAggregationSelection.bind(this);
        this.sendPivotTableUpdateMessage = this.sendPivotTableUpdateMessage.bind(this);
        // If there is any data to pivot, create a new blank sheet for this pivot table
        if (props.sheetJSONArray.length !== 0) {
            this.sendPivotTableUpdateMessage(false);
        }
    }
    /*
        Returns the remaining possible column header keys.
        A column header can only be used in either the row or column key
    */
    getSelectableKeyColumnHeaders() {
        const allColumnHeaders = this.props.sheetJSONArray[this.state.selectedSheetIndex].columns;
        return allColumnHeaders.filter(x => !this.state.selectedGroupByRows.includes(x) && !this.state.selectedGroupByCols.includes(x));
    }
    render() {
        /*
            If there is no possible Pivot taskpane that can be displayed (e.g. the sheetJSON is empty),
            give an error message indicating so.
        */
        if (this.props.sheetJSONArray.length === 0) {
            return (react_1.default.createElement(DefaultTaskpane_1.default, { header: 'Create a Pivot Table', setCurrOpenTaskpane: this.props.setCurrOpenTaskpane, taskpaneBody: react_1.default.createElement(react_1.Fragment, null, "Please Import data before pivoting.") }));
        }
        const selectableColumnHeaders = this.getSelectableKeyColumnHeaders();
        const groupByRows = this.state.selectedGroupByRows;
        const groupByCols = this.state.selectedGroupByCols;
        return (react_1.default.createElement(DefaultTaskpane_1.default, { header: 'Create a Pivot Table', setCurrOpenTaskpane: this.props.setCurrOpenTaskpane, taskpaneBody: react_1.default.createElement(react_1.Fragment, null,
                react_1.default.createElement("div", { className: 'default-taskpane-body-section-div pivot-taskpane-section-header-div' },
                    react_1.default.createElement("p", { className: 'default-taskpane-body-section-title-text' }, "Data Source"),
                    react_1.default.createElement(LargeSelect_1.default, { startingValue: this.state.dfNames[this.state.selectedSheetIndex], key: this.state.selectedSheetIndex, optionsArray: this.state.dfNames, setValue: this.setSelectedSheet })),
                react_1.default.createElement("div", { className: 'default-taskpane-body-section-div' },
                    react_1.default.createElement(PivotTableKeySelection_1.default, { sectionTitle: 'Rows', selectableColumnHeaders: selectableColumnHeaders, selectedColumnHeaders: groupByRows, editKeySelection: this.editKeySelection, keyType: KeyType.Row })),
                react_1.default.createElement("div", { className: 'default-taskpane-body-section-div' },
                    react_1.default.createElement(PivotTableKeySelection_1.default, { sectionTitle: 'Columns', selectableColumnHeaders: selectableColumnHeaders, selectedColumnHeaders: groupByCols, editKeySelection: this.editKeySelection, keyType: KeyType.Col })),
                react_1.default.createElement("div", { className: 'default-taskpane-body-section-div' },
                    react_1.default.createElement(PivotTableValueSelection_1.default, { columnHeaders: this.props.sheetJSONArray[this.state.selectedSheetIndex].columns, aggregatedValuesInSection: this.state.values, editValueAggregationSelection: this.editValueAggregationSelection }))) }));
    }
}
exports.default = PivotTaskpane;
//# sourceMappingURL=PivotTaskpane.js.map